#include "wBLRegion.h"
#include "wClosures.h"
#include "wViscMesh.h"
#include "wInvBnd.h"
#include <iostream>
#include <vector>
#include <cmath>

using namespace dart;

BLRegion::BLRegion(double _xtrF)
{
    if (_xtrF < 0. && _xtrF != -1.)
    {
        std::cout << "Unrecognized transition location " << _xtrF << std::endl;
        throw;
    }
    xtrF = _xtrF;
    closSolver = new Closures();
    mesh = new ViscMesh();
    invBnd = new InvBnd();

    transMarker = mesh->GetnMarkers();
    xtr = 1.0;

} // end BLRegion

BLRegion::~BLRegion()
{
    delete closSolver;
    delete mesh;
    // std::cout << "~BLRegion()\n";
} // end ~BLRegion

void BLRegion::SetMesh(std::vector<double> x,
                       std::vector<double> y,
                       std::vector<double> z)
{

    unsigned int nMarkers = x.size();

    mesh->SetGlob(x, y, z);

    /* Resize and reset all variables if needed */

    if (mesh->GetDiffnElm()!=0)
    {
        U.resize(nVar * nMarkers, 0.);
        Ret.resize(nMarkers, 0.);
        Cd.resize(nMarkers, 0.);
        Cf.resize(nMarkers,0.);
        Hstar.resize(nMarkers,0.);
        Hstar2.resize(nMarkers,0.);
        Hk.resize(nMarkers,0.);
        Cteq.resize(nMarkers,0.);
        Us.resize(nMarkers,0.);
        DeltaStar.resize(nMarkers,0.);
        Delta.resize(nMarkers,0.);
        Regime.resize(nMarkers,0.);
    }
} // End SetMesh

void BLRegion::SetStagBC(double Re)
{
    double dv0 = (invBnd->GetVt(1) - invBnd->GetVt(0)) / (mesh->GetLoc(1) - mesh->GetLoc(0));

    if (dv0 > 0)
    {
        U[0] = sqrt(0.075 / (Re * dv0)); // Theta
    }                                    // End if
    else
    {
        U[0] = 1e-6; // Theta
    }                // end else
    U[1] = 2.23;     // H
    U[2] = 0;        // N
    U[3] = invBnd->GetVt(0); // ue
    U[4] = 0;        // Ctau

    Ret[0] = U[3] * U[0] * Re;

    if (Ret[0] < 1)
    {
        Ret[0] = 1;
        U[3] = Ret[0] / (U[0] * Re);
    } // End if

    //std::cout << "name " << name << " dv0 " << dv0 << " U3 " << U[3] << " U[0] " << U[0] << " \n" << std::scientific;

    DeltaStar[0] = U[0] * U[1];

    std::vector<double> ClosParam = closSolver->LaminarClosures(U[0], U[1], Ret[0], invBnd->GetMe(0), name);

    Hstar[0] = ClosParam[0];
    Hstar2[0] = ClosParam[1];
    Hk[0] = ClosParam[2];
    Cf[0] = ClosParam[3];
    Cd[0] = ClosParam[4];
    Delta[0] = ClosParam[5];
    Cteq[0] = ClosParam[6];
    Us[0] = ClosParam[7];

} // End SetStagBC

void BLRegion::SetWakeBC(double Re, std::vector<double> UpperCond, std::vector<double> LowerCond)
{

    U[0] = UpperCond[0] + LowerCond[0];                                        /* (Theta_up+Theta_low). */
    U[1] = (UpperCond[0] * UpperCond[1] + LowerCond[0] * LowerCond[1]) / U[0]; /* ((Theta*H)_up+(Theta*H)_low)/(Theta_up+Theta_low). */
    U[2] = 9;
    U[3] = invBnd->GetVt(0);                                                           /* Inviscid velocity. */
    U[4] = (UpperCond[0] * UpperCond[4] + LowerCond[0] * LowerCond[4]) / U[0]; /* ((Ct*Theta)_up+(Theta)_low)/(Ct*Theta_up+Theta_low). */

    Ret[0] = U[3] * U[0] * Re; /* Reynolds number based on the momentum thickness. */

    if (Ret[0] < 1)
    {
        Ret[0] = 1;
        U[3] = Ret[0] / (U[0] * Re);
    }
    DeltaStar[0] = U[0] * U[1]; /* deltaStar */

    /* Laminar closures. */

    std::vector<double> ClosParam = closSolver->LaminarClosures(U[0], U[1], Ret[0], invBnd->GetMe(0), name);

    Hstar[0] = ClosParam[0];
    Hstar2[0] = ClosParam[1];
    Hk[0] = ClosParam[2];
    Cf[0] = ClosParam[3];
    Cd[0] = ClosParam[4];
    Delta[0] = ClosParam[5];
    Cteq[0] = ClosParam[6];
    Us[0] = ClosParam[7];

    for (size_t k = 0; k < mesh->GetnMarkers(); ++k)
    {
        Regime[k] = 1;
    }
}

void BLRegion::ComputeBlwVel()
{
    DeltaStar[0] = U[0] * U[1];
    Blowing.resize(mesh->GetnMarkers()-1, 0.);
    for (size_t iPoint = 1; iPoint<mesh->GetnMarkers(); ++iPoint)
    {
        DeltaStar[iPoint] = U[iPoint*nVar + 0] * U[iPoint*nVar + 1];
        Blowing[iPoint-1] = (invBnd->GetRhoe(iPoint) * invBnd->GetVt(iPoint) * DeltaStar[iPoint]
                            - invBnd->GetRhoe(iPoint-1) * invBnd->GetVt(iPoint-1) * DeltaStar[iPoint-1])/(invBnd->GetRhoe(iPoint)*(mesh->GetLoc(iPoint) - mesh->GetLoc(iPoint-1)));
    }
}


void BLRegion::PrintSolution(size_t iPoint) const
{
    std::cout << "\nPoint " << iPoint << " \n"
              << std::scientific;
    std::cout << "Theta " << U[iPoint * nVar + 0] << " \n"
              << std::scientific;
    std::cout << "H " << U[iPoint * nVar + 1] << " \n"
              << std::scientific;
    std::cout << "N " << U[iPoint * nVar + 2] << " \n"
              << std::scientific;
    std::cout << "ue " << U[iPoint * nVar + 3] << " \n"
              << std::scientific;
    std::cout << "Ct " << U[iPoint * nVar + 4] << " \n"
              << std::scientific;
    std::cout << "Regime " << Regime[iPoint] << " \n"
              << std::scientific;
}